<?php
/******************************************************************************
 * $Id: config.php,v 1.3 2006/02/07 17:36:01 pspencer Exp $
 ******************************************************************************
 * Copyright (c) 2005, DM Solutions Group Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ******************************************************************************
 * configuring ka-Map! ... each section has its own description below 
 *
 * basic system configuration
 *
 * this version of ka-Map! is configured to use ka-Cache! by default.  ka-Cache!
 * uses PHP/MapScript and the PHP GD extension to render tiles, and uses 
 * PHP/MapScript to generate initialization parameters a legend, and a keymap 
 * from the selected map file.
 *
 * It should be relatively easy to replace ka-Cache! with any suitable backend
 * script written using some other flavour of MapScript or compiled code.
 *
 * Make sure to set the correct module names for your PHP extensions.
 *
 * WINDOWS USERS: you will likely need to use php_gd2.dll instead of php_gd.dll
 */
$szPHPMapScriptModule = 'php_mapscript_48.'.'dll';
$szPHPGDModule = 'php_gd2.'.'dll';

/****************************************************************************** 
 * tile generation parameters
 *
 * ka-Cache! generates tiles to load in the client application by first 
 * rendering larger areas from the map file and then slicing them up into 
 * smaller tiles. This approach reduces the overhead of loading PHP/MapScript 
 * and PHP GD and drawing the map file.  These larger areas are referred to as
 * metaTiles in the code.  You can set the size of both the small tiles and the
 * metaTiles here.  A reasonable size for the small tiles seems to be 200 
 * pixels square. Smaller tiles seem to cause problems in client browsers by 
 * causing too many images to be created and thus slowing performance of live 
 * dragging.  Larger tiles take longer to download to the client and are 
 * inefficient.
 *
 * The number of smaller tiles that form a metaTile can also be configured.
 * This parameter allows tuning of the tile generator to ensure optimal
 * performance and for label placement.  MapServer will produce labels only
 * within a rendered area.  If the area is too small then features may be
 * labelled multiple times.  If the area is too large, it may exceed MapServer's
 * maximum map size (by default 2000x2000) or be too resource-intensive on the
 * server, ultimately reducing performance.
 *
 * TODO: 
 * PS: experiment with different values and measure the effect on performance
 * PS: can we build this to run off a fastCGI mapserver to avoid caching
 *     altogether?  Not for high-performance sites but for 'applications' that
 *     have a small number of users, it may be worthwhile trying.
 */
$tileWidth = 300;
$tileHeight =300;
$metaWidth = 4;
$metaHeight = 4;
/* $metaBuffer = Buffer size in pixels to add around metatiles to avoid 
 * rendering issues along the edge of the map image.  You need mapserver
 * 4.6 final for this to work - it was introduced about a month before
 * the final release so some cvs and beta versions may have it.
 */
$metaBuffer = 10;  
    
/****************************************************************************** 
 * in-image debugging information - tile location, outlines etc.
 * to use this, you need to remove images from your cache first.  This also
 * affects the meta tiles - if debug is on, they are not deleted.
 *
 * TODO: 
 * PS: add more options to fine-tune debugging information?
 */
$bDebug = false;
 
/****************************************************************************** 
 * aszMapFiles - an array of map files available to the application.  How this
 * is used is determined by the application.  Each map file is entered into this
 * array as a key->value pair.
 *
 * The key is the name to be used by the tile caching system to store cached
 * tiles within the base cache directory.  This key should be a single word that
 * uniquely identifies the map.
 *
 * The value associated with each key is an array of three values.  The first
 * value is a human-readable name to be presented to the user (should the
 * application choose to do so) and the second value is the path to the map
 * file.  It is assumed that the map file is fully configured for use with
 * MapServer/MapScript as no error checking or setting of values is done.  The 
 * third value is an array of scale values for zooming.
 *
 * TODO:
 * PS: could we get this information from the map file so that we'd only need
 *     a list of map files here?  We'd use metadata ... but this only works
 *     if we don't have to load all the map files to find the right one ...
 *     in the scalebar.php, legend.php and keymap.php scripts, performance
 *     could suffer.  tile.php also uses this config file so it has to be
 *     as quick as possible.
 */
 
$aszMapFiles = array(
  "gmap75" => array( "GMap 75", "/ms4w/apps/gmap/htdocs/gmap75.map",
                     array( 40000000, 25000000, 12000000, 7500000, 3000000, 1000000 ),
                      "PNG"),
);

/****************************************************************************** 
 * ka-Cache! - caching for ka-Map!
 * 
 * this is the directory within which ka-Cache! will create its tile cache.  The
 * directory does NOT have to be web-accessible, but it must be writable by the
 * web-server-user and allow creation of both directories AND files.
 *
 * the tile caching system will create a separate subdirectory within the base
 * cache directory for each map file.  Within the cache directory for each map
 * file, directories will be created for each group of layers.  Within the group
 * directories, directories will be created at each of the configured scales
 * for the application (see mapfile configuration above.)
 */
$szBaseCacheDir =  "/ms4w/tmp/ms_tmp/kacache/";

/****************************************************************************** 
 * figure out which map file to use and set up the necessary variables for
 * the rest of the code to use.  This does need to be done on every page load
 * unfortunately.
 *
 * szMap should be set to the default map file to use but can change if
 * this script is called with map=<mapname>. 
 */
$szMap = 'gmap75';

/*****  END OF CONFIGURABLE STUFF - unless you know what you are doing   *****/

if (isset($_REQUEST['map']) && isset($aszMapFiles[$_REQUEST['map']]))
{
    $szMap = $_REQUEST['map'];
}

$szMapCacheDir = $szBaseCacheDir.$szMap."/";
$szMapName = $aszMapFiles[$szMap][0];
$szMapFile = $aszMapFiles[$szMap][1];
$anScales = $aszMapFiles[$szMap][2];
setOutputFormat($aszMapFiles[$szMap][3]);

/****************************************************************************** 
 * output format of the map and resulting tiles
 *
 * The output format used with MapServer can greatly affect appearance and
 * performance.  It is recommended to use an 8 bit format such as PNG
 *
 * Using this function to select the output format allows fine-grained control
 * over which GD functions are used to handle the images.  Mixing true-colour
 * and 8 bit images results in odd results but can be done.
 *
 * TODO: add configuration of image cache headers here?  Or globally since it
 * doesn't really have to do with image formats ...
 */
function setOutputFormat($szFormat)
{
    switch(strtoupper($szFormat)) {
        case "PNG24":
            $GLOBALS['szMapImageFormat'] = 'PNG24';
            $GLOBALS['szMapImageCreateFunction'] = "imagecreatefrompng";
            $GLOBALS['szImageExtension'] = '.png';
            $GLOBALS['szImageCreateFunction'] = "imagecreatetruecolor";
            $GLOBALS['szImageOutputFunction'] = "imagepng";
            $GLOBALS['szImageHeader'] = 'image/png';
            break;
        case "GIF":
            $GLOBALS['szMapImageFormat'] = 'GIF';
            $GLOBALS['szMapImageCreateFunction'] = "imagecreatefromgif";
            $GLOBALS['szImageExtension'] = '.gif';
            $GLOBALS['szImageCreateFunction'] = "imagecreate";
            $GLOBALS['szImageOutputFunction'] = "imagegif";
            $GLOBALS['szImageHeader'] = 'image/gif';
            break;
        case "JPEG":
            $GLOBALS['szMapImageFormat'] = 'JPEG';
            $GLOBALS['szMapImageCreateFunction'] = "imagecreatefromjpeg";
            $GLOBALS['szImageExtension'] = '.jpg';
            $GLOBALS['szImageCreateFunction'] = "imagecreatetruecolor";
            $GLOBALS['szImageOutputFunction'] = "imagejpeg";
            $GLOBALS['szImageHeader'] = 'image/jpeg';
            break;
        case "PNG":
            $GLOBALS['szMapImageFormat'] = 'PNG';
            $GLOBALS['szMapImageCreateFunction'] = "imagecreatefrompng";
            $GLOBALS['szImageExtension'] = '.png';
            $GLOBALS['szImageCreateFunction'] = "imagecreate";
            $GLOBALS['szImageOutputFunction'] = "imagepng";
            $GLOBALS['szImageHeader'] = 'image/png';
            break;
    }
}

/**
 * create all directories in a directory tree - found on the php web site
 * under the mkdir function ...
 */
function makeDirs($strPath, $mode = 0777)
{
   return is_dir($strPath) or ( makeDirs(dirname($strPath), $mode) and mkdir($strPath, $mode) );
}

/**
 * This function replaces all special characters in the given string.
 *
 * @param szString string - The string to convert.
 *
 * @return string converted
 */
function normalizeString($szString)
{
    // Normalize string by replacing all special characters
    // e.g.    "http://my.host.com/cgi-bin/mywms?"
    // becomes "http___my_host_com_cgi_bin_mywms_"
    return preg_replace("/(\W)/", "_", $szString);
}
?>
